function [amp,phase,RscVar] = readhgt(infile,subset);

%
%   readfile        - reads roi_pac, doris, gamma, IREA, ... file types
%
% usage:  [a,p,rscinfo] = readfile(infile);
%    or:  [a,p,rscinfo] = readfile(infile,subset);
% 
%
%         extracts subset if given: 
%                  subset=[ymin xmin ypix xpix]
%         for subset=struct('YXLW',[100 100 200 300],'Proj','CART or GEO')  data(100:300,100:400) are extracted
%
% Example [a,p]=readhgt('/RAID1/ngourmelen/DONE_N_Lanari_T442/GEOsimFiles/geo_991111-000921.sim_4rlks.unw');
%
%
% NG, August 07  
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

infile=deblank(infile);extens=infile(size(infile,2)-2:size(infile,2));
Rscfile=[infile,'.rsc'];
RscVar=ReadKeywordfile([infile,'.rsc']);

if (strcmp(extens,'cor') | strcmp(extens,'unw') | strcmp(extens,'hgt'))

	fid=fopen(infile,'r');
	[F,count]=fread(fid,'float32');
	fclose(fid);

	nlines   = length(F)/(2*RscVar.WIDTH);
	recl     = 8 * RscVar.WIDTH;

	phase = zeros(nlines,RscVar.WIDTH);
	amp = zeros(nlines,RscVar.WIDTH);

	for line = 1:nlines,
 		amp(line,:)  = F((line-1)*(2*RscVar.WIDTH)+1:(line-1)*(2*RscVar.WIDTH)+RscVar.WIDTH)';
 		phase(line,:)= F((line-1)*(2*RscVar.WIDTH)+RscVar.WIDTH+1 : line*(RscVar.WIDTH*2))';
	end

elseif (strcmp(extens,'int') | strcmp(extens,'slc'))

	fid=fopen(infile,'r');
	[F,count]=fread(fid,'float'); 
    
    indx=[1:2:size(F,1)];indy=[2:2:size(F,1)];
    amp=sqrt(F(indx).*F(indx)+F(indx).*F(indx));
    pha=atan2(F(indy),F(indx));
    pha=reshape(pha,RscVar.WIDTH,RscVar.FILE_LENGTH);
    amp=reshape(amp,RscVar.WIDTH,RscVar.FILE_LENGTH);
	phase=flipud(rot90(pha));amp=flipud(rot90(amp));
    

elseif (strcmp(extens,'dem'))

	fid=fopen(infile,'r');
	[dem]=fread(fid,[RscVar.WIDTH RscVar.FILE_LENGTH],'short');
	demflr=fliplr(dem);amp=rot90(demflr);

else

error(sprintf('%s: exiting --- Unknown file type'));

end

if exist('subset')

	[x1,y1]=meshgrid(1:size(phase,2),1:size(phase,1)) ;

	subset.x=[subset.x subset.x(:,1)];subset.y=[subset.y subset.y(:,1)];  points=[NaN,NaN];

  	if isfield(subset,'Proj')==0

		for ni=1:size(subset.x,1)
        	in=inpolygon(x1(:),y1(:),subset.x(ni,:),subset.y(ni,:));  points=[points(:,1),points(:,2);y1(in),x1(in)];
    	end

    	RscVar.X_FIRST=RscVar.X_FIRST+(min(points(:,2))-1)*RscVar.X_STEP;RscVar.Y_FIRST=RscVar.Y_FIRST+(min(points(:,2))-1)*RscVar.Y_STEP;

  	elseif strcmp(subset.Proj,'GEO')==1

    	for ni=1:size(subset.x,1)

			locind=LL2ind_hgt(infile,[subset.y(ni,:)' subset.x(ni,:)']);	
			in=inpolygon(x1(:),y1(:),locind(:,2),locind(:,1));  points=[points(:,1),points(:,2);y1(in),x1(in)];
			xall(:,ni)=locind(:,2);yall(:,ni)=locind(:,1);

		end

    	junk=subset.x(find(xall'==min(min(xall))));  RscVar.X_FIRST=junk(1);
		junk=subset.y(find(yall'==min(min(yall))));  RscVar.Y_FIRST=junk(1);

  	else
  	
		for ni=1:size(subset.x,1)
            in=inpolygon(x1(:),y1(:),subset.x(ni,:),subset.y(ni,:));  points=[points(:,1),points(:,2);y1(in),x1(in)];
        end

        RscVar.X_FIRST=RscVar.X_FIRST+(min(points(:,2))-1)*RscVar.X_STEP;RscVar.Y_FIRST=RscVar.Y_FIRST+(min(points(:,2))-1)*RscVar.Y_STEP;

	end

	points(1,:)=[];
	
	L=YX2L(points(:,1),points(:,2),size(phase,1));hh=1:size(phase,1)*size(phase,2);hh(L)=[];

   	phase(hh)=NaN; phase=phase(min(points(:,1)):max(points(:,1)),min(points(:,2)):max(points(:,2)));
	amp(hh)=NaN; amp=amp(min(points(:,1)):max(points(:,1)),min(points(:,2)):max(points(:,2)));

  	RscVar.WIDTH=size(phase,2);RscVar.FILE_LENGTH=size(phase,1);

end





