// given three lines of input float data, return the slope of the
// center line of data. Data must exceed the maskmin

#include "constants.h"
#include <math.h>
#include <iostream.h>

template <class kind>
void taspect(kind *indat, float *outdat, int ns, float maskmin, float pixdim) ;

template <class kind>
void tslope (kind *indat, float *outdat, int ns, float maskmin, float pixdim) ;

void slope (unsigned char * indat, float *outdat, int ns, int dtype, float pixdim)
{
	if (dtype==0)
		tslope ((unsigned char *) indat,  outdat, ns, (float) -50., pixdim) ; 
	if (dtype==1)
		tslope ((short *) indat,  outdat, ns, (float) -50., pixdim) ; 
	if (dtype==2)
		tslope ((int *) indat,  outdat, ns, (float) -50., pixdim) ; 
	if (dtype==3)
		tslope ((float *) indat, outdat, ns, (float) -50., pixdim) ; 
}



template <class kind>
void tslope (kind *indat, float *outdat, int ns, float maskmin, float pixdim) 
{
	
	int 	i, j, npts_top, npts_bot, npts_lft, npts_rht ;
	kind	*kdatptr ;
	float   *outptr ;
	double  fval0, fval1, vertslope, horizslope, sum, tval ;
	double  bval, lval, rval, slope ;

	for (i=0; i<ns; i++) *(outdat+i) = 0. ;

	for (i=1; i<ns-1; i++)
	{
		// get the sum of the top three pixels
		kdatptr = indat + i - ns - 1 ;
		outptr = outdat + i ;
		npts_top = 0 ;
		sum = 0. ;
		for (j=0; j<3; j++) 
		{
			fval0 = (double) *(kdatptr + j) ;
			if (fval0 > maskmin)
			{
				npts_top++ ;
				sum += fval0 ;
			}
		}
		if (npts_top)
			tval = sum / npts_top ;
		else 
		{ 
			*outptr = 0. ;
			continue ;
		}
		// now get the average bottom value
		npts_bot = 0 ;
		sum = 0. ;
		kdatptr = indat + i + ns - 1 ;
		for (j=0; j<3; j++) 
		{
			fval0 = (double) *(kdatptr + j) ;
			if (fval0 > maskmin)
			{
				npts_bot++ ;
				sum += fval0 ;
			}
		}
		if (npts_bot)
			bval = sum / npts_bot ;
		else 
		{ 
			*outptr = 0. ;
			continue ;
		}

		// now get the average left value
		npts_lft = 0 ;
		sum = 0. ;
		kdatptr = indat + i - ns - 1 ;
		for (j=0; j<3; j++) 
		{
			fval0 = (double) *(kdatptr + j * ns) ;
			if (fval0 > maskmin)
			{
				npts_lft++ ;
				sum += fval0 ;
			}
		}
		if (npts_lft)
			lval = sum / npts_bot ;
		else 
		{ 
			*outptr = 0. ;
			continue ;
		}

		// now get the average rightt value
		npts_rht = 0 ;
		sum = 0. ;
		kdatptr = indat + i - ns + 1 ;
		for (j=0; j<3; j++) 
		{
			fval0 = (double) *(kdatptr + j * ns) ;
			if (fval0 > maskmin)
			{
				npts_rht++ ;
				sum += fval0 ;
			}
		}
		if (npts_rht)
			rval = sum / npts_rht ;
		else 
		{ 
			*outptr = 0. ;
			continue ;
		}

		// if we get to this point there is a valid point
		// for each of the directions
		// now do the slope calcs 
		vertslope = (tval - bval)/(pixdim * 2.) ;
		horizslope = (lval - rval) /(pixdim * 2.) ; 
		slope = pow(vertslope,2.) + pow(horizslope,2.) ;
		slope = sqrt(slope) ;
		// the slope is now percent slope
		// convert to degree slope 
		slope = atan (slope) ;
		slope = slope * RAD2DEG ;
		*outptr = slope ;
		
	}

}

template <class kind>
void taspect(kind *indat, float *outdat, int ns, float maskmin, float pixdim) 
{
	
	int 	i, j, npts_top, npts_bot, npts_lft, npts_rht ;
	kind	*kdatptr ;
	float   *outptr ;
	double  fval0, fval1, vertslope, horizslope, sum, tval ;
	double  bval, lval, rval, slope, aspect ;

	for (i=0; i<ns; i++) *(outdat+i) = 0. ;
	for (i=1; i<ns-1; i++)
	{
		// get the sum of the top three pixels
		kdatptr = indat + i - ns - 1 ;
		outptr = outdat + i ;
		npts_top = 0 ;
		sum = 0. ;
		for (j=0; j<3; j++) 
		{
			fval0 = (double) *(kdatptr + j) ;
			if (fval0 > maskmin)
			{
				npts_top++ ;
				sum += fval0 ;
			}
		}
		if (npts_top)
			tval = sum / npts_top ;
		else 
		{ 
			*outptr = 0. ;
			continue ;
		}
		// now get the average bottom value
		npts_bot = 0 ;
		sum = 0. ;
		kdatptr = indat + i + ns - 1 ;
		for (j=0; j<3; j++) 
		{
			fval0 = (double) *(kdatptr + j) ;
			if (fval0 > maskmin)
			{
				npts_bot++ ;
				sum += fval0 ;
			}
		}
		if (npts_bot)
			bval = sum / npts_bot ;
		else 
		{ 
			*outptr = 0. ;
			continue ;
		}

		// now get the average left value
		npts_lft = 0 ;
		sum = 0. ;
		kdatptr = indat + i - ns - 1 ;
		for (j=0; j<3; j++) 
		{
			fval0 = (double) *(kdatptr + j * ns) ;
			if (fval0 > maskmin)
			{
				npts_lft++ ;
				sum += fval0 ;
			}
		}
		if (npts_lft)
			lval = sum / npts_bot ;
		else 
		{ 
			*outptr = 0. ;
			continue ;
		}

		// now get the average rightt value
		npts_rht = 0 ;
		sum = 0. ;
		kdatptr = indat + i - ns + 1 ;
		for (j=0; j<3; j++) 
		{
			fval0 = (double) *(kdatptr + j * ns) ;
			if (fval0 > maskmin)
			{
				npts_rht++ ;
				sum += fval0 ;
			}
		}
		if (npts_rht)
			rval = sum / npts_rht ;
		else 
		{ 
			*outptr = 0. ;
			continue ;
		}

		// if we get to this point there is a valid point
		// for each of the directions
		// now do the slope calcs 
		vertslope = (tval - bval)/(pixdim * 2.) ;
		horizslope = (lval - rval) /(pixdim * 2.) ; 
		if (vertslope != 0.0) 
		{
			aspect = atan (horizslope/ vertslope) * RAD2DEG ;
		if (vertslope > 0.)
			aspect += 180. ;
		if (vertslope < 0. && horizslope>0)
			aspect += 360. ;
		}
		if (vertslope==0.)
		{
			if (horizslope <0) aspect = 90. ;
			if (horizslope >0) aspect = 270. ;
			if (horizslope ==0) aspect = 400. ;
		}
		// the slope is now percent slope
		// convert to degree slope 
		*outptr = aspect ;
		
	}

}


			
void aspect (unsigned char * indat, float *outdat, int ns, int dtype, float pixdim)
{
	
	if (dtype==0)
		taspect((unsigned char *) &indat[0],  outdat, ns, float (-50.), pixdim) ; 
	if (dtype==1)
		taspect((short *) indat,  outdat, ns, (float) -50., pixdim) ; 
	if (dtype==2)
		taspect ((int *) indat,  outdat, ns, (float)-50., pixdim) ; 
	if (dtype==3)
		taspect ((float *) indat, outdat, ns, (float) -50., pixdim) ; 
}
